// HUtilityImage.h : interface of the HUtilityImage class


#ifndef _HUTILITYIMAGE_H
#define _HUTILITYIMAGE_H

#include "HTools.h"
#include <stdio.h>

class MVO_API HUtilityImage
{
public:
	/*!
		The datatype for an RGB pixel.
	*/
	typedef struct {unsigned char r, g, b;} Pixel;

	static bool ImageToTiff( HC_KEY image_key,const char *filename);
	static bool GifToImage(char *filename, char *imagename,char *format, HC_KEY &image_key);

private:
// TIFF utility datastructures and methods 
	typedef struct {				// Fixed header
	  unsigned short ByteOrder;		// Intel or Motorola byte ordering
	  unsigned short Version;		// TIFF version identifier
	  unsigned long  Offset;		// Offset to first Image File
	} TIFFHEADER;					// Directory (IFD)

	typedef struct {				// A tag entry
	  unsigned short Tag;			// Tag identifier
	  unsigned short DataType;		// Data type of value stored in entry
	  unsigned long Length;			// Number of values in tag entry
	  unsigned long Offset;			// Offset or value of tag
	} TIFFTAGENTRY;

	typedef struct {				// Values read from the tag entries
	  short BitsPerSample;			// 8 for grayscale; 8,8,8 for RGB
	  short Compression;			// Compression flag
	  short SamplesPerPixel;		// 1 for grayscale; 3 for RGB
	  short NumStrips;				// Number of strips in file
	  unsigned long *StripLengths;	// Array of strip lengths
	  unsigned long *StripOffsets;	// Array of strip file offsets
	} TIFFIMAGEINFO;

	int ReadStrips(FILE *fp, TIFFIMAGEINFO *ImInfo,unsigned char  *Im);
	int ReadGrayTIFFImage(FILE *fp, unsigned char  *Im);
	int ReadRGBTIFFImage(FILE *fp, unsigned char  *Im);
	int GetTIFFImageInfo(FILE *fp, int *ImWd, int *ImHt, int *ImType);
	int GetImageInfo(char *Path, int *ImWd, int *ImHt, int *ImType);
	int ReadImage(char *Path, unsigned char  *Im);
	static int WriteTag(FILE *fp, unsigned int Tag, unsigned int DataType, unsigned long Len, unsigned long Offs);
	static int WriteRGBTIFFHeader(FILE *fp, unsigned long ImWd, unsigned long ImHt, unsigned long StripSize);
	int WriteGrayTIFFHeader(FILE *fp, unsigned long ImWd, unsigned long ImHt, unsigned long StripSize);
	static int WriteFxdHdr(FILE *fp);
	void SwapWordOrder(unsigned short *Word);
	void SwapLongOrder(unsigned long *Long);
	int ReadTag(FILE *fp, TIFFTAGENTRY *Entry);
	int ReadWord(FILE *fp, unsigned short *Val);
	int ReadULong(FILE *fp, unsigned long *Val);
	int ReadTags(FILE *fp, TIFFIMAGEINFO *ImInfo);
	int ReadFxdHdr(FILE *fp, unsigned long *FileOffs);

	void fput_word(FILE	*fp, int	n);
	void fput_long(FILE *fp, long n);
	void write_TIFF_tag(FILE *fp, int tag, int type, long length, long offset);
	void image_to_TIFF (HC_KEY image_key, FILE	*fp);

	unsigned char IntelOrdr;


// GIF utility datastructures and methods
	typedef  struct list {
		int	prefix,
		suffix;
    } LIST;

	typedef	 struct color {
		unsigned char	red;
		unsigned char   green;
		unsigned char   blue;
    } COLOR;

	typedef  struct	global_header {
		char signature[6];					/* Should be GIF87a */
		unsigned short	screen_width;		/* Size of saver's screen */
		unsigned short	screen_height;		/* Size of saver's screen */
		unsigned char	flags;				/* Global map, etc. */
		unsigned char	background_color;	/* Should be able to ignore */
		unsigned char	trailer;			/* Should always be 0 */
    } GLOBAL_HEADER;


 
	typedef  struct	local_header {
		unsigned short	image_left;			/* Where to place the image */
											/* We ignore this. */
		unsigned short	image_top;			/* Where to place the image */
											/* We ignore this. */
		unsigned short	image_width;		/* Size of the image */
		unsigned short	image_height;		/* Size of the image */
		unsigned char	flags;				/* Local map, etc. */
		unsigned char	lzw_code_size;		/* Bits for lzw */
    } LOCAL_HEADER;

 
	 typedef struct decoder_data {
		FILE	*fp;
		bool	failed;
		bool	use_gif_format;
		int		bits_left;
		int		bits_per_code;
		unsigned char	*gif_block_pos;
		unsigned char	*block_pos;
		unsigned char	*block_end;
		unsigned char	*raw_block;
		long	prev_code;
		unsigned int	clear_code;
		unsigned int	EOI_code;
		unsigned int	*prefix;
		unsigned char	*suffix;
		unsigned int	*length;
		unsigned int	highest_defined_code;
		unsigned long	code_mask;
		unsigned long	current_bits;
		long	total_unpacked_bytes;
		int		data_bit_size;
		int		max_bits_per_code;
		int		max_legal_code;
		long	leftover_data_length;
		unsigned char	*leftover_data;
		unsigned char	*leftover_place;
	} DECODER_DATA;

	static void *start_unpacking_from_file (FILE *fp,bool use_gif_format,int data_bit_size, int max_bits_per_code);
	static void finish_unpacking_from_file(void *u_data);
	static bool unpack_from_file(void *u_data, long num_bytes, unsigned char *out_buffer, long	*bytes_unpacked);
	static void Show_Error (char *message);	
};

#endif
